package com.dzeio.openhealth.data.step

import android.content.Context
import android.hardware.Sensor
import android.hardware.SensorEvent
import android.hardware.SensorEventListener
import android.hardware.SensorManager
import android.os.SystemClock
import android.util.Log
import androidx.preference.PreferenceManager
import com.dzeio.openhealth.Application
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.runBlocking

class StepSource(
    private val context: Context,
    private val callback: ((Float) -> Unit)? = null
): SensorEventListener {

    companion object {
        const val TAG = "${Application.TAG}/StepSource"
    }

    private val prefs = PreferenceManager.getDefaultSharedPreferences(context)

    private var timeSinceLastRecord: Long
        get() {
            return prefs.getLong("steps_time_since_last_record", Long.MAX_VALUE)
        }
        set(value) {
            val editor = prefs.edit()
            editor.putLong("steps_time_since_last_record", value)
            editor.commit()
        }
    private var stepsAsOfLastRecord: Float
        get() {
            return prefs.getFloat("steps_as_of_last_record", 0f)
        }
        set(value) {
            val editor = prefs.edit()
            editor.putFloat("steps_as_of_last_record", value)
            editor.commit()
        }


    init {
        Log.d(TAG, "Setting up")
        val sensorManager = context.getSystemService(Context.SENSOR_SERVICE) as SensorManager
        val stepCountSensor = sensorManager.getDefaultSensor(Sensor.TYPE_STEP_COUNTER)
        stepCountSensor.let {
            sensorManager.registerListener(this, it, SensorManager.SENSOR_DELAY_NORMAL, SensorManager.SENSOR_DELAY_NORMAL)
            Log.d(TAG, "should be setup :D")
        }
    }

    val events = Channel<Float>(10)

    override fun onSensorChanged(ev: SensorEvent?) {
        if (ev == null) {
            return
        }
        //        Log.d(TAG, "Sensor changed: $ev")
        ev.values.firstOrNull()?.let {
            val timeSinceLastBoot = System.currentTimeMillis() - SystemClock.elapsedRealtime()

            val diff = it - stepsAsOfLastRecord
            stepsAsOfLastRecord = it

            // don't send changes since it wasn't made when the app was running
            if (timeSinceLastBoot < timeSinceLastRecord) {
                Log.d(TAG, "Skipping since we don't know when many steps are taken since last boot ($timeSinceLastRecord, $timeSinceLastBoot)")
                timeSinceLastRecord = timeSinceLastBoot
                return@let
            }

            timeSinceLastRecord = timeSinceLastBoot

            runBlocking {
                // keep check because shit as already happened where it send an event even before it is set
                if (events != null) {
                    events.send(diff)
                }
            }
            callback?.invoke(diff)
        }
    }

    override fun onAccuracyChanged(sensor: Sensor?, accuracy: Int) {
        Log.d(TAG, "[Accuracy changed]: Sensor: $sensor, Accuracy: $accuracy")
    }
}
