package com.dzeio.openhealth.ui.home

import android.content.SharedPreferences
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.dzeio.openhealth.Settings
import com.dzeio.openhealth.core.BaseViewModel
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.data.water.WaterRepository
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.data.weight.WeightRepository
import com.dzeio.openhealth.units.UnitFactory
import com.dzeio.openhealth.units.Units
import com.dzeio.openhealth.utils.Configuration
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class HomeViewModel @Inject internal constructor(
    private val weightRepository: WeightRepository,
    private val waterRepository: WaterRepository,
    settings: SharedPreferences,
    config: Configuration
) : BaseViewModel() {

    private val _water = MutableLiveData<Water?>(null)
    val water: LiveData<Water?> = _water

    private val _weights = MutableLiveData<List<Weight>?>(null)
    val weights: LiveData<List<Weight>?> = _weights

    var waterCupSize = config.getInt("water_cup_size").toLiveData()

    var waterUnit =
        UnitFactory.volume(settings.getString("water_unit", "milliliter") ?: "Milliliter")

    private val _massUnit = MutableLiveData(Units.Mass.KILOGRAM)
    val massUnit: LiveData<Units.Mass> = _massUnit

    val goalWeight = config.getFloat(Settings.WEIGHT_GOAL).toLiveData()

    val dailyWaterIntake: Int =
        ((settings.getString("water_intake", "1200")?.toFloatOrNull() ?: 1200f) * waterUnit.modifier)
            .toInt()

    init {
        viewModelScope.launch {
            waterRepository.todayWater().collectLatest {
                _water.postValue(it)
            }
        }

        viewModelScope.launch {
            weightRepository.getWeights().collectLatest {
                _weights.postValue(it)
            }
        }

        config.getString(Settings.MASS_UNIT).apply {
            addObserver {
                if (it == null) return@addObserver
                _massUnit.postValue(Units.Mass.find(it))
            }
            if (value != null) {
                _massUnit.postValue(Units.Mass.find(value!!))
            }
        }
    }

    /**
     * @deprecated
     */
    fun fetchWeights() = weightRepository.getWeights()

    /**
     * @deprecated
     */
    fun lastWeight() = weightRepository.lastWeight()

    fun fetchWeight(id: Long) = weightRepository.getWeight(id)

    suspend fun deleteWeight(weight: Weight) = weightRepository.deleteWeight(weight)

    suspend fun addWeight(weight: Weight) = weightRepository.addWeight(weight)

    fun fetchTodayWater() = waterRepository.todayWater()

    fun updateWater(water: Water) {
        viewModelScope.launch {
            waterRepository.addWater(water)
        }
    }

    fun deleteWater(item: Water) {
        viewModelScope.launch {
            waterRepository.deleteWater(item)
            _water.postValue(null)
        }
    }
}
