package com.dzeio.openhealth.utils

import android.content.SharedPreferences
import android.util.Log
import androidx.core.content.edit
import com.dzeio.openhealth.Application
import com.dzeio.openhealth.core.Observable

class Configuration(
    private val prefs: SharedPreferences
) : SharedPreferences.OnSharedPreferenceChangeListener {

    private val cache: HashMap<String, Field<*>> = HashMap()

    private companion object {
        const val TAG = "${Application.TAG}/Config"
    }

    init {
        prefs.registerOnSharedPreferenceChangeListener(this)
    }

    fun getString(key: String): StringField {
        if (cache[key] == null) {
            Log.d(TAG, "$key does not exist in cache, creating new instance")
            cache[key] = StringField(key)
        } else {
            Log.d(TAG, "$key in cache")
        }
        return cache[key] as StringField
    }

    fun getLong(key: String): LongField {
        if (cache[key] == null) {
            Log.d(TAG, "$key does not exist in cache, creating new instance")
            cache[key] = LongField(key)
        } else {
            Log.d(TAG, "$key in cache")
        }
        return cache[key] as LongField
    }

    fun getBoolean(key: String): BooleanField {
        if (cache[key] == null) {
            Log.d(TAG, "$key does not exist in cache, creating new instance")
            cache[key] = BooleanField(key)
        } else {
            Log.d(TAG, "$key in cache")
        }
        return cache[key] as BooleanField
    }

    fun getInt(key: String): IntField {
        if (cache[key] == null) {
            Log.d(TAG, "$key does not exist in cache, creating new instance")
            cache[key] = IntField(key)
        } else {
            Log.d(TAG, "$key in cache")
        }
        return cache[key] as IntField
    }

    fun getFloat(key: String): FloatField {
        if (cache[key] == null) {
            Log.d(TAG, "$key does not exist in cache, creating new instance")
            cache[key] = FloatField(key)
        } else {
            Log.d(TAG, "$key in cache")
        }
        return cache[key] as FloatField
    }

    fun getStringSet(key: String): StringSetField {
        if (cache[key] == null) {
            Log.d(TAG, "$key does not exist in cache, creating new instance")
            cache[key] = StringSetField(key)
        } else {
            Log.d(TAG, "$key in cache")
        }
        return cache[key] as StringSetField
    }

    override fun onSharedPreferenceChanged(u: SharedPreferences, key: String) {
        Log.d(TAG, "configuration update for key: $key")
        cache[key]?.needUpdate = true
        cache[key]?.notifyObservers()
    }

    abstract class Field<T>(
        baseValue: T
    ) : Observable<T>(baseValue) {
        abstract fun exists(): Boolean
        abstract fun internalGet(): T
        abstract fun internalSet(value: T)
        var needUpdate = true

        override var value: T = baseValue
            get() {
                if (needUpdate) {
                    field = if (!exists()) {
                        null as T
                    } else {
                        internalGet()
                    }
                }
                return field
            }
            set(value) {
                if (field == value) {
                    return
                }
                field = value
                internalSet(value)
                notifyObservers()
            }
    }

    inner class StringField(
        private val key: String,
        private val defaultValue: String? = null
    ) : Field<String?>(defaultValue) {
        override fun exists(): Boolean = prefs.contains(key)
        override fun internalGet(): String? = prefs.getString(key, defaultValue)
        override fun internalSet(value: String?) =
            prefs.edit { if (value == null) remove(key) else putString(key, value) }
    }

    inner class BooleanField(
        private val key: String,
        private val defaultValue: Boolean = false
    ) : Field<Boolean?>(defaultValue) {
        override fun exists(): Boolean = prefs.contains(key)
        override fun internalGet(): Boolean = prefs.getBoolean(key, defaultValue)
        override fun internalSet(value: Boolean?) =
            prefs.edit { if (value == null) remove(key) else putBoolean(key, value) }
    }

    inner class IntField(
        private val key: String,
        private val defaultValue: Int = -1
    ) : Field<Int?>(defaultValue) {
        override fun exists(): Boolean = prefs.contains(key)
        override fun internalGet(): Int = prefs.getInt(key, defaultValue)
        override fun internalSet(value: Int?) =
            prefs.edit { if (value == null) remove(key) else putInt(key, value) }
    }

    inner class FloatField(
        private val key: String,
        private val defaultValue: Float = -1f
    ) : Field<Float?>(defaultValue) {
        override fun exists(): Boolean = prefs.contains(key)
        override fun internalGet(): Float = prefs.getFloat(key, defaultValue)
        override fun internalSet(value: Float?) =
            prefs.edit { if (value == null) remove(key) else putFloat(key, value) }
    }

    inner class LongField(
        private val key: String,
        private val defaultValue: Long = -1
    ) : Field<Long?>(defaultValue) {
        override fun exists(): Boolean = prefs.contains(key)
        override fun internalGet(): Long = prefs.getLong(key, defaultValue)
        override fun internalSet(value: Long?) =
            prefs.edit { if (value == null) remove(key) else putLong(key, value) }
    }

    inner class StringSetField(
        private val key: String,
        private val defaultValue: MutableSet<String>? = null
    ) : Field<MutableSet<String>?>(defaultValue) {
        override fun exists(): Boolean = prefs.contains(key)
        override fun internalGet(): MutableSet<String>? = prefs.getStringSet(key, defaultValue)
        override fun internalSet(value: MutableSet<String>?) =
            prefs.edit { if (value == null) remove(key) else putStringSet(key, value) }
    }
}
